package com.sun.dft.glassfish.http.failover.ejbref;

import java.io.IOException;
import java.util.StringTokenizer;

import org.testng.Assert;
import org.testng.annotations.AfterTest;
import org.testng.annotations.BeforeTest;
import org.testng.annotations.Test;

import com.meterware.httpunit.GetMethodWebRequest;
import com.meterware.httpunit.WebConversation;
import com.meterware.httpunit.WebRequest;
import com.meterware.httpunit.WebResponse;
import com.sun.dft.glassfish.config.BaseTestCase;
import com.sun.dft.glassfish.helpers.FailureInjectionService;
import com.sun.dft.glassfish.utils.TestUtils;
import com.sun.dft.glassfish.helpers.HttpMessageLogger;

public class EJBRefFO16_restart extends BaseTestCase {

    private String servingInstance;
	
    @BeforeTest
    protected void setUp() throws Exception {
		asadminHelper.deployWithLBReconfig("web/ejbref/SSOFormApp.ear", "SSOFormWeb", TestUtils.getDefaultClusterName());
	        checkPoint("Deployed SSOFormApp");
    }

    @Test(description="Failover of session(with restart) with SSO and EJB refs " +
		" stored in the session. ")
    public void EJBRefFO16Test1_restart() throws Exception {
		

                // Creating the users and setting up SSO
                asadminHelper.deleteFileUser("j2ee");
                asadminHelper.createFileUser("j2ee", "secret", "staff");

                String clusterName = TestUtils.getDefaultClusterName();

                // asadminHelper.set(clusterName + ".http-service.virtual-server.server.sso-enabled", "true");
                //checkPoint("Enabling sso in virtual server");

                //  asadminHelper.set(clusterName + "-config.availability-service.web-container-availability.sso-failover-enabled", "true");
                //checkPoint("Set availability-service.web-container-availability.sso-failover-enabled=true");


                //asadminHelper.stopCluster();
                //asadminHelper.startCluster();
                //checkPoint("Restarting cluster - " + clusterName);

                String webserverHostPort = webserverHelper.getHost() + ":" + webserverHelper.getHttpPort();
		String request1 = "/SSOFormWeb/SSOServlet";
		String requestString = "http://" + webserverHostPort + request1;

		checkPoint("** Sending Request to Webserver " + webserverHostPort + "> " + request1);
		WebConversation webConversation = new WebConversation();
		WebRequest webRequest = new GetMethodWebRequest(requestString);
		WebResponse response = webConversation.getResponse(webRequest);

                // where to see this message from logHttpRequestResponse
		logHttpRequestResponse(response);
                // checkPoint("dbg:logging response for "+ requestString + " in framwork.log");
                HttpMessageLogger.logHttpRequestResponse(webConversation, webRequest, response);

                String request2 = "/SSOFormWeb/j_security_check?j_username=j2ee&j_password=secret";
		String newRequestString = "http://" + webserverHostPort + request2;

		checkPoint("** Sending Request to Webserver " + webserverHostPort + "> " + request2);
		WebRequest newWebRequest = new GetMethodWebRequest(newRequestString);



		WebResponse newResponse = webConversation.getResponse(newWebRequest);

		logHttpRequestResponse(newResponse);
                // checkPoint("dbg: logging response for "+ newRequestString + " in framework.log");
                HttpMessageLogger.logHttpRequestResponse(webConversation, newWebRequest, newResponse);

		String servingInstance = findServingInstance(newResponse);

		checkPoint("** Serving Instance (1st Request): " + servingInstance);

		checkPoint("** Killing serving instance...");
		new FailureInjectionService().killInstance(servingInstance);

                // TODO: Use TestUtils to wait for message that indicates unhealthy instance.
		TestUtils.sleepSeconds(10);
		checkPoint("** Starting serving instance...");
		asadminHelper.startInstance(servingInstance);

                checkPoint("** Sending Request to Webserver " + webserverHostPort + "> " + request1);
		WebRequest newWebRequest3 = new GetMethodWebRequest(requestString);


		WebResponse newResponse3 = webConversation.getResponse(newWebRequest3);
              
		logHttpRequestResponse(newResponse3);
                // checkPoint("dbg: log response for "+ requestString + " in framework.txt");
                HttpMessageLogger.logHttpRequestResponse(webConversation, newWebRequest3, newResponse3);

		String newServingInstance = findServingInstance(newResponse3);


		checkPoint("** Serving Instance (2nd Request): " + newServingInstance);
		
		
		boolean sessionContainsRetrievedData = newResponse.getText().contains("Successfully called validate method in SSOBean");
		checkPoint("** Session data contains EJBRef and works with SSO! " + sessionContainsRetrievedData);

                Assert.assertTrue(sessionContainsRetrievedData,
				"Session data has the EJBRef");


    }

    // TODO: Check: This is perhaps better done by checking server logs ? That way it will be reusable ?
    private String  findServingInstance(WebResponse response) 
       throws IOException {
    	String responseText = response.getText();
    	StringTokenizer tokenizer = new StringTokenizer(responseText, "\n");
    	while (tokenizer.hasMoreTokens()){
    		String token = tokenizer.nextToken();
    		if (token.contains("Executed From Instance:")){
    			String instanceName = token.substring(token.indexOf("<b>")+3, token.indexOf("</b>"));
    			logger.info("Serving Instance: " + instanceName);
    			return instanceName;
    		}
    	}
    	String errorMessage = "Unable to find serving Instance from the response text";
    	logger.severe(errorMessage);
		throw new RuntimeException(errorMessage);
    }
    
    public static synchronized void logHttpRequestResponse(WebResponse response) 
	throws IOException {
        System.out.println("-------------Request/Response : START ------------");
        System.out.println(response);
        System.out.println(response.getText());
        System.out.println("-------------Request/Response : END --------------");
    }

    @AfterTest
    public void tearDown(){
	// asadminHelper.startCluster();
	asadminHelper.undeploy("SSOFormApp", TestUtils.getDefaultClusterName());
	checkPoint("Undeployed SSOFormApp");
    }
}
